
    #	Process time lapse movie frames and assemble into movie

    use strict;
    use warnings;
    
    use Getopt::Long;
    
    my $version = '1.0';
    
#    my $title = "Clouds above Fourmilab, 2004-09-06";
    my $title = "Les Quatre Saisons, Lignieres 2005-2006";
    my $credits = "John Walker, www.fourmilab.ch";
    my $copyright = "This movie is in the public domain";
    
    #	Show verbose processing information ?    
    my $verbose = 0;
   
    my $jpeg_job = 0;  	    	    # Make JPEG frames
    my $avi_job = 0;   	    	    # Encode JPEG frames as AVI movie
    my $whack_job = 1;   	    # Encode grabbed frames as AVI movie
    
    my $grab = 'registered';	    # Directory containing raw frame grabs
    my $jpegdir = "jpeg";   	    # JPEG frame directory
    
#    my ($framewidth, $frameheight) = (320, 240);    # Fit frames into this size
    my ($framewidth, $frameheight) = (640, 480);    # Fit frames into this size
#    my ($framewidth, $frameheight) = (800, 532);    # Fit frames into this size
#    my ($framewidth, $frameheight) = (0, 0);    # Use frames at existing size
    my $framerate = 1;	    	    # Frames per second
#    my $preprocess = 'ppmnorm';	    # Preprocessing for raw frames
    my $preprocess = '';	    # Preprocessing for raw frames
    my $postprocess = '';   	    # Postprocessing for scaled frames
    
    my $video_codec = '-ovc lavc -lavcopts vcodec=msmpeg4v2'; # Video CODEC and options
#$video_codec = '-ovc lavc -ffourcc DX50';

#    The -ffourcc DX50 option is included because although the video
#    produced is an MPEG4, the default header is FMP4 which is not widely
#    recognised. A header with DX50 claims that the video is DivX 5 (hence
#    MPEG4) compatible and should therefore play on any media player that
#    can play MPEG4 video.
#   	http://gentoo-wiki.com/HOWTO_Mencoder_Introduction_Guide
    
    #	Process command line options
    
    my $ook = GetOptions(
	       "copyright" => \&infoOption,
	       "help" => \&infoOption,
	       "verbose" => \$verbose,
	       "version" => \&infoOption
	       );
	       
    if (!$ook) {
    	exit(1);    	    # Command line syntax error
    }

    
    #	Utilities invoked by this program may be downloaded from the
    #	following locations:
    
    #	NETPBM Image Processing Toolkit
    #	    http://netpbm.sourceforge.net/
    
    #	MPLAYER/MENCODER Movie Player/Encoder for Linux
    #	    http://www.mplayerhq.hu/
    

    #	Prepare JPEG frames from cropped images
        
    if ($jpeg_job) {
    
    	command("rm $jpegdir/*.jpg\n");
	opendir(CD, $grab);
	my @frames = grep(/\.(JPG|PNG)$/i, readdir(CD));
	closedir(CD);
	@frames = sort(@frames);
	
	if ($preprocess ne '') {
	    $preprocess .= ' | ';
	}
	
	if ($postprocess ne '') {
	    $postprocess .= ' | ';
	}

    	my ($vfn, $nfiles) = (0, 0);
	my $f;
	foreach $f (@frames) { 
	    $vfn++;
	    my $input = "$grab/$f";
	    my $base = $f;
	    $base =~ s/\.(JPG|PNG)//i;
	    my $filetype = $1;
	    my $decoder = ($filetype eq 'jpg') ? 'jpegtopnm' : 'pngtopnm';
	    my $output = $base;
	    $output =~ tr/A-Z/a-z/;
	    $output = "$jpegdir/$output.png";

	    print("$input -> $output\n");
	    
	    my $scale = ($framewidth == 0) ? '' : "pnmscale -xysize $framewidth $frameheight |";
	    
	    my $c;
	    if (($scale eq '') && ($preprocess eq '') && ($postprocess eq '')) {
	    	$c = "cp -p $input $output\n";
	    } else {
	    	$c = "$decoder $input | $preprocess" .
    	    	     "$scale $postprocess".
	    	     "pnmtopng >$output\n";
	    }
	    command($c);

	    $nfiles++;
#if ($nfiles >= 10) {
#    last;
#}
	}
    }
    
    #	Encode JPEG frames and MP3 sound track as AVI movie
     
     if ($avi_job) {
	my $menc = 
		   "-mf fps=$framerate -ovc lavc -lavcopts vcodec=msmpeg4v2 " .
		   "-info 'name=$title:artist=$credits:copyright=$copyright' " .
		   "-o movie.avi";


	command("mencoder \"mf://$jpegdir/*.png\" $menc\n");
     }
     
     #	Encode grabbed frames as AVI movie
     
     if ($whack_job) {
	my $menc = 
    	    	   "-audiofile music/seasons/seasons.wav -oac mp3lame " .
		   "-mf fps=$framerate $video_codec " .
		   "-info 'name=$title:artist=$credits:copyright=$copyright' " .
		   "-o movie.avi";


	command("mencoder \"mf://$grab/*.png\" $menc\n");
     }
    
    #	Print a command and then run it
    
    sub command {
    	my ($c) = @_;
	
	print($c);
	system($c);
    }
  
    #	Process information request options
    
    sub infoOption {
    	my ($opt) = @_;
	
	if ($opt eq'copyright') {
	    print("This program is in the public domain.\n");
	    exit(0);
	}
	
	if ($opt eq 'help') {
	    print << "EOD";
usage: time_lapse [ options ]

Options:
  --copyright              Display copyright information
  --help                   Display this message
  --outputfile fname       Output file name (default standard output)
  --verbose                Display verbose status information
  --version                Display version number
  
Please report bugs to bugs\@fourmilab.ch; include "time_lapse"
in the Subject line.
EOD
	    
	    exit(0);
	}
	
	if ($opt eq 'version') {
	    print("$version\n");
	    exit(0);
	}

    }
